//+------------------------------------------------------------------+
//|                                                    BBflat_sw.mq4 |
//|                                                          by Raff |
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers     4
#property indicator_color1      clrNONE
#property indicator_color2      clrRed
#property indicator_color3      clrRed
#property indicator_color4      clrLime

//----
input int                period          = 9;
input int                shift           = 0;
input ENUM_MA_METHOD     method          = MODE_SMA;
input ENUM_APPLIED_PRICE price           = 0;
input double             deviation       = 1.5;
input bool               alertsOn        = true;           // Turn alerts on/off?
input bool               alertsOnCurrent = false;          // Alerts on open bar on/off?
input bool               alertsMessage   = true;           // Alerts message on/off?
input bool               alertsSound     = false;          // Alerts sound on/off?
input bool               alertsNotify    = false;          // Alerts notification on/off?
input bool               alertsEmail     = false;          // Alerts email on/off?
input string             soundFile       = "alert2.wav";   // Sound file

double buffer1[],buffer2[],buffer3[],buffer4[],vala[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
//
//
//

int OnInit()
{
   IndicatorBuffers(5);
   SetIndexBuffer(0,buffer1); SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(1,buffer2); SetIndexStyle(1,DRAW_LINE);
   SetIndexBuffer(2,buffer3); SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(3,buffer4); SetIndexStyle(3,DRAW_LINE);
   SetIndexBuffer(4,vala);

return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int  OnCalculate(const int rates_total,const int prev_calculated,const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
{
   int i=rates_total-prev_calculated+1; if (i>=rates_total) i=rates_total-1; 
   
   //
   //
   //
   
  for (; i>=0 && !_StopFlag; i--) 
  {
      double ima = iMA(NULL,0,period,shift,method,price,i);
      double std = deviation*iStdDev(NULL,0,period,shift,method,price,i);
      buffer1[i] = 0;
      buffer2[i] = std;
      buffer3[i] = -std;
      buffer4[i] = close[i]-ima;
      vala[i] = (buffer4[i]>buffer2[i]) ? 1 : (buffer4[i]<buffer3[i]) ? -1 : 0; 
   }
   manageAlerts(); 
return(rates_total);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (vala[whichBar] != vala[whichBar+1])
      {
         if (vala[whichBar] ==  1) doAlert(whichBar,"crossed upper band");
         if (vala[whichBar] == -1) doAlert(whichBar,"crossed lower band");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];
       
       //
       //
       //
       //
       //

       message = timeFrameToString(_Period)+" "+_Symbol+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" BBflat "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsNotify)  SendNotification(message);
          if (alertsEmail)   SendMail(_Symbol+" BBflat ",message);
          if (alertsSound)   PlaySound(soundFile);
      }
}
